import argparse
import sys
import torch.nn.functional as F
import torch
from tqdm import tqdm
import numpy as np

sys.path.append('..')
sys.path.append('.')

parser = argparse.ArgumentParser(description='Syn')
parser.add_argument('--device', type=int)
parser.add_argument('--epoch', type=int,default=20,help="Number of epochs to train.")
parser.add_argument('--modalities', type=int, default=3, help='Number of modalities')
parser.add_argument('--classes', type=int, default=2, help='Number of classes')
parser.add_argument('--split', type=int, default=4000, help='Number of feature split')
parser.add_argument('--labeled', type=int, default=500, help='Number of labeled data')
parser.add_argument('--thresh', type=float, default=0.9, help='Threshold')
parser.add_argument('--tcgm', default=False, action='store_true', help='MIG or not')
parser.add_argument('--cluster', default=False, action='store_true', help='cluster or not')
parser.add_argument('--s', default=False, action='store_true')
parser.add_argument('--hgr', default=False, action='store_true', help='HGR or not')
parser.add_argument('--vat', type=int, default=0, help='VAT or not')
parser.add_argument('--alpha', type=float, default=1.0, help='hyper-parameters for MIG')
parser.add_argument('--lam', type=float, default=0.5, help='hyper-parameters for HGR')
parser.add_argument('--xi', type=float, default=0.01, help='VAT perturbation( best hyper is default)')
parser.add_argument('--f', type=float, default=0.1, help='label fraction')
parser.add_argument('--ratio', type=float, default=0.1, help='label fraction')
parser.add_argument('--lr', type=float, default=1e-2, help='Learning rate')
parser.add_argument('--data', type=str, help='dataset')
parser.add_argument('--seed', type=int, help='random seed')

args = parser.parse_args()
args.cnt = 0
args.interval = 120
from dataset import *
from model import *
from utils import mig_loss_function,enu_modalities,get_index_3,mig_loss_function_multi, hgr_loss_function

best_val_acc = np.zeros(args.modalities+1)
best_test_acc = 0
best_test_acc_all = 0
best_val_acc_all = 0
best_test_acc_hgr = 0


def cotrain(epoch, device):

    for t in range(epoch):

        acc = np.zeros((args.modalities), dtype=np.float32)
        train_loss = 0.0
        for idx, (data_dict,label) in enumerate(dataloader['train']):

            label = label.long().to(device)
            for i in range(1, 1 + args.modalities):
                Scheduler[i].step()
                Optimizer[i].zero_grad()
                output = Net[i](data_dict[i].to(device))
                loss = F.nll_loss(output, label)
                loss.backward()
                Optimizer[i].step()

                pred = torch.max(output, 1)[1]
                acc[i-1] += pred.eq(label.data).cpu().sum()
                train_loss += loss.data
        data_collect = {}
        if split + args.cnt + args.interval < 5000:
            for i in range(args.modalities):
                data_collect[i+1] = view_train[i+1][split + args.cnt : split + args.cnt + args.interval]

            label_collect = label[split + args.cnt : split + args.cnt + args.interval]
            args.cnt += args.interval
            T_train.update_data(data_collect,label_collect, args.modalities)
            dataloader['train'] = torch.utils.data.DataLoader(dataset = T_train, batch_size = 32)
        acc /= float(len(dataloader['train'].dataset))


def test(epoch,device):

    acc = 0.0
    acc_ = np.zeros(args.modalities+1)
    data_collect = np.zeros((args.modalities, len(dataloader['test'].dataset),2))
    label_collect = np.zeros((args.modalities,len(dataloader['test'].dataset)))
    cnt = 0
    for idx, (data_dict,label) in enumerate(dataloader['test']):
        label = label.long().to(device)
        output_total = 0.0
        for i in range(1, 1 + args.modalities):
            output = Net[i](data_dict[i].to(device))
            output_total += output
        pred = torch.max(output_total, 1)[1]
        acc += pred.eq(label.data).cpu().sum()

        for i in range(1, 1 + args.modalities):
            pred = torch.max(Net[i](data_dict[i].to(device)), 1)[1]
            acc_[i] += pred.eq(label.data).cpu().sum()
        for i in range(args.modalities):
            data_collect[i][cnt : cnt + len(label)] = data_dict[i+1].data.cpu().numpy()
            pred = torch.max(Net[i+1](data_dict[i+1].to(device)), 1)[1]
            label_collect[i][cnt : cnt + len(label)] = pred.data.cpu().numpy()
        cnt += len(label)
    for i in range(args.modalities):
        np.save('data' + str(i) + '.npy', data_collect[i])
        np.save('label' + str(i) + '.npy', label_collect[i])

    acc = float(acc) / float(len(dataloader['test'].dataset))
    return acc_/float(len(dataloader['test'].dataset)), acc


def val(epoch,device):

    acc = 0.0
    acc_ = np.zeros(args.modalities+1)
    for idx, (data_dict,label) in enumerate(dataloader['val']):

        label = label.long().to(device)

        output_total = 0.0
        for i in range(1, 1 + args.modalities):

            output = Net[i](data_dict[i].to(device))
            output_total += output

        pred = torch.max(output_total, 1)[1]
        acc += pred.eq(label.data).cpu().sum()
        for i in range(1, 1 + args.modalities):
            pred = torch.max(Net[i](data_dict[i].to(device)), 1)[1]
            acc_[i] += pred.eq(label.data).cpu().sum()

    acc = float(acc) / float(len(dataloader['val'].dataset))
    return acc_ / float(len(dataloader['val'].dataset)), acc

def MIG_train_U(epoch, device,alpha):

    train_loss = 0.0
    p = [1.0/float(args.classes) for i in range(args.classes)]
    p = torch.FloatTensor(p).to(device)
    n = int((args.modalities * (args.modalities - 1)) /2)
    acc = np.zeros((n),dtype=np.float32)
    pairs = enu_modalities(args.modalities)
    output_dict = dict()
    d = get_index_3(32)
    for data_dict, label in dataloader['mig']:
        if len(label) != 32:
            continue
        for i in range(1, 1 + args.modalities):
            Optimizer[i].zero_grad()
            output = Net[i](data_dict[i].to(device), mig=True)
            output_dict[i] = output

        loss = -alpha * mig_loss_function_multi(output_dict=output_dict, modal_num=args.modalities, p=p, bs=len(label), device=device, sampling_num=1000, d=d)
        #loss = -alpha * mig_loss_function(output_dict[1],output_dict[2], p, len(label),device)
        loss.backward()
        train_loss += loss.data
        for i in range(1, 1 + args.modalities):
            Optimizer[i].step()

        output_dict.clear()


def train(epoch, device):

    for t in range(epoch):

        acc = np.zeros((args.modalities), dtype=np.float32)
        train_loss = 0.0
        for idx, (data_dict,label) in enumerate(dataloader['train']):

            label = label.long().to(device)
            for i in range(1, 1 + args.modalities):
                Scheduler[i].step()
                Optimizer[i].zero_grad()
                output = Net[i](data_dict[i].to(device))
                loss = F.nll_loss(output, label)
                loss.backward()
                Optimizer[i].step()

                pred = torch.max(output, 1)[1]
                acc[i-1] += pred.eq(label.data).cpu().sum()
                train_loss += loss.data
        acc /= (float(len(dataloader['train'].dataset))+1)
        if not args.s:
            MIG_train_U(t, device, args.alpha)
        global best_val_acc
        global best_test_acc
        global best_test_acc_all
        global best_val_acc_all
        # val_acc, val_total_acc = val(t, device)
        # for i in range(1,args.modalities+1):
        #
        #     if val_acc[i] > best_val_acc[i]:
        #         best_val_acc[i] = val_acc[i]
        #         acc_, acc = test(t,device)
        #         best_test_acc[i] = acc_[i]
        #         print("i: Best test acc:{}, Best all acc:{}".format(best_test_acc[i], best_test_acc_all))
        #     if val_total_acc > best_val_acc_all:
        #         best_val_acc_all = val_total_acc
        #         acc_, acc = test(t, device)
        #         best_test_acc_all = max(best_test_acc_all, acc)
        acc_, acc = test(t, device)
        acc = max(acc, 1.- acc)
        best_test_acc = max(best_test_acc,acc)
        print("Epoch:{}, Acc:{}".format(t, best_test_acc))
    print("Final acc:", best_test_acc)
    print("Final all acc:", best_test_acc_all)


def test_hgr(epoch,device):

    acc = 0.0
    for i in range(1, 1 + args.modalities):
        Net[i].eval()
    for idx, (data_dict,label) in enumerate(dataloader['test']):

        label = label.long().to(device)
        output_collect = []
        for i in range(1, 1 + args.modalities):
            output = Net[i](data_dict[i].to(device), hgr=True)
            output_collect.append(output)

        feature = torch.cat(output_collect, dim=1)
        output = H_Linear(feature)
        pred = torch.max(output, 1)[1]
        acc += pred.eq(label.data).cpu().sum()

    acc = float(acc) / float(len(dataloader['test'].dataset))
    return acc


def HDR_train(epoch,device):

    for t in range(epoch):

        acc = np.zeros((args.modalities), dtype=np.float32)
        train_loss = 0.0
        pairs = enu_modalities(args.modalities)
        for i in range(1, 1 + args.modalities):
            Net[i].train()
        for data_dict, label in dataloader['mig']:
            for idx, (i, j) in enumerate(pairs):
                ldata, rdata = data_dict[i].to(device), data_dict[j].to(device)
                label = label.long().to(device)

                Optimizer[i].zero_grad()
                Optimizer[j].zero_grad()
                output_i = Net[i](ldata,hgr=True)
                output_j = Net[j](rdata,hgr=True)
                loss = args.lam * hgr_loss_function(output_i,output_j,device=device)
                loss.backward()
                Optimizer[i].step()
                Optimizer[j].step()

        for data_dict, label in dataloader['train']:
            output_collect = []
            label = label.long().to(device)
            for i in range(1, 1 + args.modalities):

                Optimizer[i].zero_grad()
            H_Optimizer.zero_grad()

            for i in range(1, 1 + args.modalities):
                output = Net[i](data_dict[i].to(device),hgr = True)
                output_collect.append(output)

            feature = torch.cat(output_collect,dim=1)
            output = H_Linear(feature)
            loss = F.nll_loss(output, label)
            loss.backward()
            pred = torch.max(output, 1)[1]
            acc += pred.eq(label.data).cpu().sum().numpy()
            train_loss += loss.data

            for i in range(1, 1 + args.modalities):
                Optimizer[i].step()
            H_Optimizer.step()

        print("Training acc:",acc/float(len(dataloader['train'].dataset)))

        global best_test_acc_hgr
        acc = test_hgr(t, device)
        best_test_acc_hgr = max(best_test_acc_hgr,acc)
        print("Test accuracy:",best_test_acc_hgr)


if __name__ == '__main__':

    if args.tcgm or args.s:
        train(args.epoch, args.device)
    elif args.hgr:
        HDR_train(args.epoch, args.device)


