# -*- coding: utf-8 -*-

import random
import numpy as np
from os import listdir
from os.path import join
import scipy.misc as misc
import torch.utils.data as data
import os
import torch

# from PIL import Image
import cv2
from util.rotation_3D import rotation3d_itk
import time
import scipy
from util.utils import *
from dataset.mat_reader import Big_dict
from torch.autograd import Variable


# print 'dataset.py, ', np.random.randn(1), torch.randn(1)

def get_all_contour_list(seg_mask, theshold=0.5):
    seg_mask = seg_mask.astype(np.uint8)
    ret, binary_seg_mask = cv2.threshold(seg_mask, theshold, 255, cv2.THRESH_BINARY)
    binary_seg_mask_unit8 = binary_seg_mask.astype(np.uint8)
    _, contours_cv2, hierarchy = cv2.findContours(binary_seg_mask_unit8, cv2.RETR_TREE, cv2.CHAIN_APPROX_NONE)
    all_contour = []
    if contours_cv2 != []:
        i_max = 0
        i_index = 0

        for i in range(len(contours_cv2)):
            cnt_area = cv2.contourArea(contours_cv2[i])
            if cnt_area >= 1:
                all_contour.append(contours_cv2[i])

    return all_contour


def plot_contour_mask(img, mask):
    contour = get_all_contour_list(mask)
    plot_img = cv2.drawContours(cv2.cvtColor(img.astype('uint8'), cv2.COLOR_GRAY2RGB),
                                contour, -1, (255, 0, 0), thickness=1)
    return plot_img


def load_img_perm(use_perm, directory, patch_size, W):
    datatype = 'Raw'
    dirname = directory + '%s_%d/%s_%d/IM%04d/' % (
        datatype, patch_size, datatype, W, use_perm + 1)
    data_x = np.zeros((patch_size, patch_size, patch_size))
    for j in range(patch_size):
        try:
            filename = dirname + '%02d.png' % (j + 1)
            data_x[:, :, j] = misc.imread(filename).reshape(patch_size, patch_size)
        except:
            filename = dirname + '%02d.jpg' % (j + 1)
            data_x[:, :, j] = misc.imread(filename).reshape(patch_size, patch_size)
    return data_x


def load_img(filepath):
    img = misc.imread(filepath)
    img_shape = img.shape[1]
    img_3d = img.reshape((img_shape, img_shape, img_shape))
    img_3d = img_3d.transpose((1, 2, 0))
    return img_3d


def load_img_path(directory):
    if directory.endswith('.png'):
        return load_img(directory)
    patch_size = 96
    dirname = directory
    data_x = np.zeros((patch_size, patch_size, patch_size))
    for j in range(patch_size):
        try:

            filename = os.path.join(dirname, '%02d.png' % (j + 1))
            # print(filename)
            data_x[:, :, j] = misc.imread(filename).reshape(patch_size, patch_size)
        except:
            filename = os.path.join(dirname, '%02d.png' % (j + 1))
            data_x[:, :, j] = misc.imresize(misc.imread(filename), (patch_size, patch_size))
    return data_x


def load_label_perm(use_perm, directory, patch_size, W):
    datatype = 'Raw'
    dirname = directory + '%s_%d/%s_%d_mask/IM%04d/' % (
        datatype, patch_size, datatype, patch_size, use_perm + 1)
    data_x = np.zeros((patch_size, patch_size, patch_size))
    for j in range(patch_size):
        try:
            filename = dirname + '%02d.png' % (j + 1)
            data_x[:, :, j] = misc.imread(filename).reshape(patch_size, patch_size)
        except:
            filename = dirname + '%02d.jpg' % (j + 1)
            data_x[:, :, j] = misc.imread(filename).reshape(patch_size, patch_size)

    return data_x


def random_pick(seq, prob):
    x = random.uniform(0, 1)
    cumulative_probability = 0.0
    for item, item_probability in zip(seq, prob):
        cumulative_probability += item_probability
        if x < cumulative_probability:
            return item

class DatasetFromPerm_txt_boosting_30T(data.Dataset):
    def __init__(self, pair_image_list, crop_size=64, load_size=96, crop_mode='center', flip=False, transpose=False,
                 scale=1 / 255.0, shift=2, aug=0, is_seg=0, is_class=1, channel_num=1, W=600, W2=40,
                 random_mask=False, random_mask_size=16, noise=0, concat=0, scaling=0, MS_list=None,
                 edge_target=0, offset_max=10, merge_label=0, attr_id=8, norm=0, pair_type=1,
                 rotation=1, opt=None):
        super(DatasetFromPerm_txt_boosting_30T, self).__init__()

        with open(pair_image_list) as f:
            lines = f.readlines()

        min_num = 100
        for line in lines:
            if len(line.split(' ')) <= min_num:
                min_num = len(line.split(' '))

        self.dict = Big_dict
        self.norm = norm
        self.attr_id = attr_id
        self.rotation = rotation
        self.image_filenames = []
        for line in lines:
            self.image_filenames.append(line.split(' ')[0])

        self.label = []
        self.raw_pred = []
        self.pid_list = []
        self.merge = merge_label

        if min_num == 14 or min_num == 5 or min_num == 6 or min_num == 23 or min_num == 16:
            self.spacing_x = [float(line.split(' ')[3]) for line in lines]
            self.spacing_z = [float(line.split(' ')[4]) for line in lines]
            for line in lines:
                one_list = []
                self.raw_pred.append(float(line.split(' ')[-1].strip('\n')))
                if min_num > 7 and is_seg == 1:
                    self.pid_list.append(line.split(' ')[0].split('/')[7])
                if merge_label:
                    if is_class == 1:
                        one_list = [convert_label(float(line.split(' ')[5 + self.attr_id].strip('\n')), self.attr_id,
                                                  merge_label)]
                    elif is_class != 9 and is_class != 18:
                        if pair_type == 1:
                            pairs = [3, 4]
                        elif pair_type == 2:
                            pairs = [4, 5]
                        elif pair_type == 3:
                            pairs = [5, 8]
                        elif pair_type == 4:
                            pairs = [4, 6]
                        elif pair_type == 5:
                            pairs = [5, 6]
                        elif pair_type == 6:
                            pairs = [4, 5, 6]
                        elif pair_type == 7:
                            pairs = [6, 8]
                        elif pair_type == 10:
                            pairs = [0, 1]
                        elif pair_type == 11:
                            pairs = range(11)

                        for idx in pairs:
                            one_list.append(
                                convert_label(float(line.split(' ')[5 + idx].strip('\n')), idx, merge_label))
                    else:
                        for idx in range(is_class):
                            # print line.split(' ')[0]
                            # print is_class, len(line.split(' '))
                            one_list.append(
                                convert_label(float(line.split(' ')[5 + idx].strip('\n')), idx, merge_label))
                else:
                    if is_class == 1:
                        one_list = [int(round(float(line.split(' ')[5 + self.attr_id].strip('\n'))))]
                    else:
                        for idx in range(is_class):
                            # here all atribute adopt int
                            one_list.append(int(round(float(line.split(' ')[5 + idx].strip('\n')))))
                if is_class > 1:
                    self.label.append([one_list])
                elif is_class == 1:
                    self.label.append([one_list[-1]])  # ??
                else:
                    self.label = []
            self.seg_filenames = [line.strip().split(' ')[1] for line in lines]

            self.label = np.array(self.label)
            if np.min(self.label) == 1:
                self.label = self.label - 1
        elif min_num == 2:
            self.spacing_x = [0.6 for line in lines]
            self.spacing_z = [0.6 for line in lines]
            self.seg_filenames = [line.strip().split(' ')[1] for line in lines]

        else:
            raise ('sub_dir file is not valid')

        self.raw_pred = np.array(self.raw_pred)
        self.W2 = W2
        self.crop_size = crop_size
        self.load_size = load_size
        self.crop_mode = crop_mode
        self.flip = flip
        self.transpose = transpose
        self.scale = scale
        self.shift = shift
        self.aug = aug
        self.is_seg = is_seg
        self.is_class = is_class
        self.channel_num = channel_num
        self.random_mask = random_mask
        self.random_mask_size = random_mask_size
        self.noise = noise
        self.concat = concat
        self.scaling = scaling
        self.weight = np.ones((len(self.image_filenames)))
        self.instance_loss = np.zeros((len(self.image_filenames)))
        self.MS_list = MS_list
        self.edge_target = edge_target
        self.offset_max = offset_max
        self.shear = 0

    def __getitem__(self, index):
        final_shape = (self.channel_num, self.crop_size, self.crop_size, self.crop_size)
        class_shape = (1, self.is_class)
        index_shape = (1, 1)
        table_shape = (1, 24)
        # print self.label[index]
        if self.aug == 1:
            input = self.scale * load_img_path(self.image_filenames[index]) #96 * 96 * 96

            if self.norm:
                input = (input - 0.5) / 0.5
            if np.where(input > 0)[0].shape[0] < 1:
                print(self.image_filenames[index])
                print('bad input image with all blank')

            if self.is_seg == 1:
                # label = self.label[index]
                target = self.scale * load_img_path(self.seg_filenames[index])
                target[target >= 0.5] = 1
                target[target <= 0.5] = 0
                if np.where(target == 1)[0].shape[0] < 1:
                    print(self.seg_filenames[index])
                    print('bad target image with all blank')

                input, target = self._aug_data_seg(input, index, target)

                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:

                    if self.is_class:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(self.label[index]).reshape(class_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(index).reshape(index_shape).astype('int'))

            else:
                if self.concat == 1:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        print('bad target image with all blank')

                    input, target = self._aug_data_seg(input, index, target)
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        print('bad target image with all blank')

                    nodule_size = compute_nodule_size(target)

                    input = self._aug_data_seg(input, index, nodule_size=nodule_size)
                    if self.merge == 8 or self.merge == 9:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('float32')), \
                               torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('int')), torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))

        else:
            input = self.scale * load_img_path(self.image_filenames[index])

            if self.norm:
                input = (input - 0.5) / 0.5
            if np.where(input > 0)[0].shape[0] < 1:
                print(self.image_filenames[index])
                print('bad input image with all blank')

            if self.crop_mode == 'center':
                crop_x = int(input.shape[0] / 2 - self.crop_size / 2)
                crop_y = int(input.shape[0] / 2 - self.crop_size / 2)
                crop_z = int(input.shape[0] / 2 - self.crop_size / 2)

                input = input[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                        crop_x: (crop_x + self.crop_size)]

                if self.concat == 1:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        raise ('bad target image with all blank')
                    target = target[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                             crop_x: (crop_x + self.crop_size)]

            if self.is_seg == 1:
                # 1. norm? or 1,0 ??  2.不用aug?
                target = self.scale * load_img_path(self.seg_filenames[index])
                target[target >= 0.5] = 1
                target[target <= 0.5] = 0
                if np.where(target == 1)[0].shape[0] < 1:
                    print(self.seg_filenames[index])
                    raise ('bad target image with all blank')

                if self.crop_mode == 'center':
                    target = target[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                             crop_x: (crop_x + self.crop_size)]

                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    if self.is_class:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(self.label[index]).reshape(class_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(index).reshape(index_shape).astype('int'))

            else:
                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    if self.merge == 8 or self.merge == 9:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('float32')), \
                               torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('int')),torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))



    def _aug_data_seg(self, input, index, target=None, nodule_size=None):
        if self.offset_max:
            offset_x, offset_y, offset_z, offset_w, offset_h, offset_d, rand_angle, rand_lr, rand_td, rand_r1 = self._get_random_params(
                self.offset_max, index)

        ran_zoom = [self.crop_size, self.crop_size, self.crop_size]
        if nodule_size is not None:
            rand_w = np.random.rand()
            rand_h = np.random.rand()
            rand_d = np.random.rand()

            if max(nodule_size) <= 3:
                offset_w = rand_w * ran_zoom[0] / 2.0
                offset_h = rand_h * ran_zoom[1] / 2.0
                offset_d = rand_d * ran_zoom[2] / 2.0
            elif max(nodule_size) >= self.crop_size / 2:
                offset_w = - rand_w * ran_zoom[0] / 2.0
                offset_h = - rand_h * ran_zoom[1] / 2.0
                offset_d = - rand_d * ran_zoom[2] / 2.0

        ran_zoom[0] = ran_zoom[0] + offset_w
        ran_zoom[1] = ran_zoom[1] + offset_h
        ran_zoom[2] = ran_zoom[2] + offset_d
        zoom = [self.crop_size / float(ran_zoom[0]), self.crop_size / float(ran_zoom[1]),
                self.crop_size / float(ran_zoom[2])]
        # zoom is bigger means minimize the image

        if self.shear:
            hyx = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hzx = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hxy = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hzy = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hxz = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hyz = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            shear = [hyx, hzx, hxy, hzy, hxz, hyz]
        else:
            shear = [0, 0, 0, 0, 0, 0]

        if self.scaling:
            # scaling_list = [2.0, 1.75, 1.5, 1.25, 1, 0.8]
            # scaling_ratio_xy = random.sample(scaling_list, 1)[0]
            # scaling_ratio_z = random.sample(scaling_list, 1)[0]

            '''
            # visualization
            if not os.path.exists('/data2/botong/Github/save_img_dataset/'):
                os.makedirs('/data2/botong/Github/save_img_dataset/')
            for i in range(input.shape[2]):
                misc.imsave('/data2/botong/Github/save_img_dataset/raw_input_%02d.png'%(i + 1), input[:,:,i])
                misc.imsave('/data2/botong/Github/save_img_dataset/raw_target_%02d.png' % (i + 1), target[:, :, i])
            '''

            # input = ndimage.zoom(input, (scaling_ratio_xy, scaling_ratio_xy, scaling_ratio_z), mode='wrap')
            # target = ndimage.zoom(target, (scaling_ratio_xy, scaling_ratio_xy, scaling_ratio_z), mode='wrap')
            if self.rotation == 1:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
            elif self.rotation == 2:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
            elif self.rotation == 3:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
            else:
                R_x, R_y, R_z = 0, 0, 0

            # print(R_x, R_y, R_z, zoom, shear)
            input = rotation3d_itk(input, R_x, R_y, R_z, zoom, shear)
            input = np.array(input)
            if target is not None:
                target = rotation3d_itk(target, R_x, R_y, R_z, zoom, shear)
                target = np.array(target)

                target[target >= 0.5] = 1.0
                target[target < 0.5] = 0.0
            scaling_ratio_x = zoom[0]
            scaling_ratio_y = zoom[1]
            scaling_ratio_z = zoom[2]

        else:
            scaling_ratio_x = 1.0
            scaling_ratio_y = 1.0
            scaling_ratio_z = 1.0

        if self.shift > 0:
            shift_x = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
            shift_y = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
            shift_z = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
        else:
            shift_x, shift_y, shift_z = 0, 0, 0

        crop_x = int(input.shape[0] / 2 - self.crop_size / 2 + shift_x)
        crop_y = int(input.shape[1] / 2 - self.crop_size / 2 + shift_y)
        crop_z = int(input.shape[2] / 2 - self.crop_size / 2 + shift_z)

        '''
                # visualization
        print('scaling ratio', scaling_ratio_xy, scaling_ratio_z)

        if not os.path.exists('/data2/botong/Github/save_img_dataset/'):
            os.makedirs('/data2/botong/Github/save_img_dataset/')
        for i in range(input.shape[2]):
            misc.imsave('/data2/botong/Github/save_img_dataset/input_%02d.png'%(i + 1), input[:,:,i])
            misc.imsave('/data2/botong/Github/save_img_dataset/target_%02d.png' % (i + 1), target[:, :, i])
        '''

        input_aug = input[crop_x: (crop_x + self.crop_size), crop_y: (crop_y + self.crop_size),
                    crop_z: (crop_z + self.crop_size)]
        if target is not None:
            target_aug = target[crop_x: (crop_x + self.crop_size), crop_y: (crop_y + self.crop_size),
                         crop_z: (crop_z + self.crop_size)]

        '''
                # visualization
        for i in range(input_aug.shape[2]):
            misc.imsave('/data2/botong/Github/save_img_dataset/input_aug_%02d.png'%(i + 1), input_aug[:,:,i])
            misc.imsave('/data2/botong/Github/save_img_dataset/target_aug_%02d.png' % (i + 1), target_aug[:, :, i])
        '''

        transpose_type = [(0, 1, 2), (1, 0, 2), (2, 0, 1)]

        if self.transpose:
            trans_idx = np.random.choice(3)
            if trans_idx != 0:
                input_aug = input_aug.transpose(transpose_type[trans_idx])
                if target is not None:
                    target_aug = target_aug.transpose(transpose_type[trans_idx])

        if self.flip:
            flip_x = np.random.choice(2) * 2 - 1
            flip_y = np.random.choice(2) * 2 - 1
            flip_z = np.random.choice(2) * 2 - 1
            input_aug = input_aug[::flip_x, ::flip_y, ::flip_z]
            if target is not None:
                target_aug = target_aug[::flip_x, ::flip_y, ::flip_z]

        if self.noise:
            input_aug = input_aug + np.random.normal(0, 0.1, (self.crop_size, self.crop_size, self.crop_size))

        if self.random_mask == 1:
            rand_x = np.random.choice(self.crop_size)
            rand_y = np.random.choice(self.crop_size)
            rand_z = np.random.choice(self.crop_size)

            input_aug[rand_x:min(rand_x + self.random_mask_size, self.crop_size - 1),
            rand_y:min(rand_y + self.random_mask_size, self.crop_size - 1),
            rand_z:min(rand_z + self.random_mask_size, self.crop_size - 1)] = input_aug.mean()

        elif self.random_mask == 2:
            rand_x = np.random.randint(self.crop_size)
            rand_y = np.random.randint(self.crop_size)
            rand_z = np.random.randint(self.crop_size)

            rand_x1 = np.clip(rand_x - self.random_mask_size / 2, 0, self.crop_size)
            rand_x2 = np.clip(rand_x + self.random_mask_size / 2, 0, self.crop_size)
            rand_y1 = np.clip(rand_y - self.random_mask_size / 2, 0, self.crop_size)
            rand_y2 = np.clip(rand_y + self.random_mask_size / 2, 0, self.crop_size)
            rand_z1 = np.clip(rand_z - self.random_mask_size / 2, 0, self.crop_size)
            rand_z2 = np.clip(rand_z + self.random_mask_size / 2, 0, self.crop_size)

            input_aug[rand_x1:rand_x2, rand_y1:rand_y2, rand_z1:rand_z2] = input_aug.mean()

        if target is not None:
            return input_aug, target_aug
        else:
            return input_aug

    def _get_random_params(self, offset_max, index=0):
        # np.random.seed(int(time.time() + 1e5 * index))

        rand_x = np.random.rand()
        rand_y = np.random.rand()
        rand_z = np.random.rand()
        rand_w = np.random.rand()
        rand_h = np.random.rand()
        rand_r = np.random.rand()
        rand_lr = np.random.rand()
        rand_td = np.random.rand()
        rand_r1 = np.random.rand()

        offset_x = int((rand_x * 2 - 1) * offset_max * 0.3)
        offset_y = int((rand_y * 2 - 1) * offset_max * 0.3)
        offset_z = int((rand_z * 2 - 1) * offset_max * 0.3)
        offset_w = int(((rand_w + 0.25) * 2 - 1) * offset_max * 2)
        offset_h = int(((rand_h + 0.25) * 2 - 1) * offset_max * 2)
        offset_d = int(((rand_h + 0.25) * 2 - 1) * offset_max * 2)
        # offset_w = int(((rand_w) * 2 - 1) * offset_max * 2)
        # offset_h = int(((rand_h) * 2 - 1) * offset_max * 2)
        rand_angle = int(rand_r * 360)

        return offset_x, offset_y, offset_z, offset_w, offset_h, offset_d, rand_angle, rand_lr, rand_td, rand_r1

    def __len__(self):
        return len(self.image_filenames)
class DatasetFromPerm_txt_boosting_(data.Dataset):
    def __init__(self, pair_image_list, crop_size=64, load_size=96, crop_mode='center', flip=False, transpose=False,
                 scale=1 / 255.0, shift=2, aug=0, is_seg=0, is_class=1, channel_num=1, W=600, W2=40,
                 random_mask=False, random_mask_size=16, noise=0, concat=0, scaling=0, MS_list=None,
                 edge_target=0, offset_max=10, merge_label=0, attr_id=8, norm=0, pair_type=1,
                 rotation=1, opt=None):
        super(DatasetFromPerm_txt_boosting, self).__init__()

        with open(pair_image_list) as f:
            lines = f.readlines()

        min_num = 100
        for line in lines:
            if len(line.split(' ')) <= min_num:
                min_num = len(line.split(' '))

        self.dict = Big_dict
        self.norm = norm
        self.attr_id = attr_id
        self.rotation = rotation
        self.image_filenames = []
        for line in lines:
            self.image_filenames.append(line.split(' ')[0])

        self.label = []
        self.raw_pred = []
        self.pid_list = []
        self.merge = merge_label

        if min_num == 14 or min_num == 5 or min_num == 6 or min_num == 23 or min_num == 16:
            self.spacing_x = [float(line.split(' ')[3]) for line in lines]
            self.spacing_z = [float(line.split(' ')[4]) for line in lines]
            for line in lines:
                one_list = []
                self.raw_pred.append(float(line.split(' ')[-1].strip('\n')))
                if min_num > 7 and is_seg == 1:
                    self.pid_list.append(line.split(' ')[0].split('/')[7])
                if merge_label:
                    if is_class == 1:
                        one_list = [convert_label(float(line.split(' ')[5 + self.attr_id].strip('\n')), self.attr_id,
                                                  merge_label)]
                    elif is_class != 9 and is_class != 18:
                        if pair_type == 1:
                            pairs = [3, 4]
                        elif pair_type == 2:
                            pairs = [4, 5]
                        elif pair_type == 3:
                            pairs = [5, 8]
                        elif pair_type == 4:
                            pairs = [4, 6]
                        elif pair_type == 5:
                            pairs = [5, 6]
                        elif pair_type == 6:
                            pairs = [4, 5, 6]
                        elif pair_type == 7:
                            pairs = [6, 8]
                        elif pair_type == 10:
                            pairs = [0, 1]
                        elif pair_type == 11:
                            pairs = range(11)

                        for idx in pairs:
                            one_list.append(
                                convert_label(float(line.split(' ')[5 + idx].strip('\n')), idx, merge_label))
                    else:
                        for idx in range(is_class):
                            # print line.split(' ')[0]
                            # print is_class, len(line.split(' '))
                            one_list.append(
                                convert_label(float(line.split(' ')[5 + idx].strip('\n')), idx, merge_label))
                else:
                    if is_class == 1:
                        one_list = [int(round(float(line.split(' ')[5 + self.attr_id].strip('\n'))))]
                    else:
                        for idx in range(is_class):
                            # here all atribute adopt int
                            one_list.append(int(round(float(line.split(' ')[5 + idx].strip('\n')))))
                if is_class > 1:
                    self.label.append([one_list])
                elif is_class == 1:
                    self.label.append([one_list[-1]])  # ??
                else:
                    self.label = []
            self.seg_filenames = [line.strip().split(' ')[1] for line in lines]

            self.label = np.array(self.label)
            if np.min(self.label) == 1:
                self.label = self.label - 1
        elif min_num == 2:
            self.spacing_x = [0.6 for line in lines]
            self.spacing_z = [0.6 for line in lines]
            self.seg_filenames = [line.strip().split(' ')[1] for line in lines]

        else:
            raise ('sub_dir file is not valid')

        self.raw_pred = np.array(self.raw_pred)
        self.W2 = W2
        self.crop_size = crop_size
        self.load_size = load_size
        self.crop_mode = crop_mode
        self.flip = flip
        self.transpose = transpose
        self.scale = scale
        self.shift = shift
        self.aug = aug
        self.is_seg = is_seg
        self.is_class = is_class
        self.channel_num = channel_num
        self.random_mask = random_mask
        self.random_mask_size = random_mask_size
        self.noise = noise
        self.concat = concat
        self.scaling = scaling
        self.weight = np.ones((len(self.image_filenames)))
        self.instance_loss = np.zeros((len(self.image_filenames)))
        self.MS_list = MS_list
        self.edge_target = edge_target
        self.offset_max = offset_max
        self.shear = 0

    def __getitem__(self, index):
        final_shape = (self.channel_num, self.crop_size, self.crop_size, self.crop_size)
        class_shape = (1, self.is_class)
        index_shape = (1, 1)
        table_shape = (1, 24)
        # print self.label[index]
        if self.aug == 1:
            input = self.scale * load_img_path(self.image_filenames[index]) #96 * 96 * 96
            T = self.image_filenames[index][36:38] #15 or 30

            if self.image_filenames[index][40] == 'M':
                #MCI
                gen = self.image_filenames[index][40:43]

                #index
                num = self.image_filenames[index][44:].find('.')
                num = self.image_filenames[index][44:][:num]
            else:
                #AD NC
                gen = self.image_filenames[index][40:42]

                #index
                num = self.image_filenames[index][43:].find('.')
                num = self.image_filenames[index][43:][:num]
            #print(T, gen, num)
            table = self.dict[T][gen][int(num)-1][:table_shape[1]]

            if self.norm:
                input = (input - 0.5) / 0.5
            if np.where(input > 0)[0].shape[0] < 1:
                print(self.image_filenames[index])
                print('bad input image with all blank')

            if self.is_seg == 1:
                # label = self.label[index]
                target = self.scale * load_img_path(self.seg_filenames[index])
                target[target >= 0.5] = 1
                target[target <= 0.5] = 0
                if np.where(target == 1)[0].shape[0] < 1:
                    print(self.seg_filenames[index])
                    print('bad target image with all blank')

                input, target = self._aug_data_seg(input, index, target)

                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:

                    if self.is_class:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(self.label[index]).reshape(class_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(index).reshape(index_shape).astype('int'))

            else:
                if self.concat == 1:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        print('bad target image with all blank')

                    input, target = self._aug_data_seg(input, index, target)
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        print('bad target image with all blank')

                    nodule_size = compute_nodule_size(target)

                    input = self._aug_data_seg(input, index, nodule_size=nodule_size)
                    if self.merge == 8 or self.merge == 9:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('float32')), \
                               torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('int')), torch.from_numpy(np.array(index).reshape(index_shape).astype('int')), torch.FloatTensor(table).reshape(table_shape)

        else:
            input = self.scale * load_img_path(self.image_filenames[index])
            T = self.image_filenames[index][36:38]  # 15 or 30
            if self.image_filenames[index][40] == 'M':
                # MCI
                gen = self.image_filenames[index][40:43]

                # index
                num = self.image_filenames[index][44:].find('.')
                num = self.image_filenames[index][44:][:num]
            else:
                # AD NC
                gen = self.image_filenames[index][40:42]

                # index
                num = self.image_filenames[index][43:].find('.')
                num = self.image_filenames[index][43:][:num]
            table = self.dict[T][gen][int(num)-1][:table_shape[1]]
            if self.norm:
                input = (input - 0.5) / 0.5
            if np.where(input > 0)[0].shape[0] < 1:
                print(self.image_filenames[index])
                print('bad input image with all blank')

            if self.crop_mode == 'center':
                crop_x = int(input.shape[0] / 2 - self.crop_size / 2)
                crop_y = int(input.shape[0] / 2 - self.crop_size / 2)
                crop_z = int(input.shape[0] / 2 - self.crop_size / 2)

                input = input[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                        crop_x: (crop_x + self.crop_size)]

                if self.concat == 1:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        raise ('bad target image with all blank')
                    target = target[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                             crop_x: (crop_x + self.crop_size)]

            if self.is_seg == 1:
                # 1. norm? or 1,0 ??  2.不用aug?
                target = self.scale * load_img_path(self.seg_filenames[index])
                target[target >= 0.5] = 1
                target[target <= 0.5] = 0
                if np.where(target == 1)[0].shape[0] < 1:
                    print(self.seg_filenames[index])
                    raise ('bad target image with all blank')

                if self.crop_mode == 'center':
                    target = target[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                             crop_x: (crop_x + self.crop_size)]

                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    if self.is_class:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(self.label[index]).reshape(class_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(index).reshape(index_shape).astype('int'))

            else:
                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    if self.merge == 8 or self.merge == 9:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('float32')), \
                               torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('int')),torch.from_numpy(np.array(index).reshape(index_shape).astype('int')),torch.FloatTensor(table).reshape(table_shape)



    def _aug_data_seg(self, input, index, target=None, nodule_size=None):
        if self.offset_max:
            offset_x, offset_y, offset_z, offset_w, offset_h, offset_d, rand_angle, rand_lr, rand_td, rand_r1 = self._get_random_params(
                self.offset_max, index)

        ran_zoom = [self.crop_size, self.crop_size, self.crop_size]
        if nodule_size is not None:
            rand_w = np.random.rand()
            rand_h = np.random.rand()
            rand_d = np.random.rand()

            if max(nodule_size) <= 3:
                offset_w = rand_w * ran_zoom[0] / 2.0
                offset_h = rand_h * ran_zoom[1] / 2.0
                offset_d = rand_d * ran_zoom[2] / 2.0
            elif max(nodule_size) >= self.crop_size / 2:
                offset_w = - rand_w * ran_zoom[0] / 2.0
                offset_h = - rand_h * ran_zoom[1] / 2.0
                offset_d = - rand_d * ran_zoom[2] / 2.0

        ran_zoom[0] = ran_zoom[0] + offset_w
        ran_zoom[1] = ran_zoom[1] + offset_h
        ran_zoom[2] = ran_zoom[2] + offset_d
        zoom = [self.crop_size / float(ran_zoom[0]), self.crop_size / float(ran_zoom[1]),
                self.crop_size / float(ran_zoom[2])]
        # zoom is bigger means minimize the image

        if self.shear:
            hyx = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hzx = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hxy = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hzy = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hxz = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hyz = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            shear = [hyx, hzx, hxy, hzy, hxz, hyz]
        else:
            shear = [0, 0, 0, 0, 0, 0]

        if self.scaling:
            # scaling_list = [2.0, 1.75, 1.5, 1.25, 1, 0.8]
            # scaling_ratio_xy = random.sample(scaling_list, 1)[0]
            # scaling_ratio_z = random.sample(scaling_list, 1)[0]

            '''
            # visualization
            if not os.path.exists('/data2/botong/Github/save_img_dataset/'):
                os.makedirs('/data2/botong/Github/save_img_dataset/')
            for i in range(input.shape[2]):
                misc.imsave('/data2/botong/Github/save_img_dataset/raw_input_%02d.png'%(i + 1), input[:,:,i])
                misc.imsave('/data2/botong/Github/save_img_dataset/raw_target_%02d.png' % (i + 1), target[:, :, i])
            '''

            # input = ndimage.zoom(input, (scaling_ratio_xy, scaling_ratio_xy, scaling_ratio_z), mode='wrap')
            # target = ndimage.zoom(target, (scaling_ratio_xy, scaling_ratio_xy, scaling_ratio_z), mode='wrap')
            if self.rotation == 1:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
            elif self.rotation == 2:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
            elif self.rotation == 3:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
            else:
                R_x, R_y, R_z = 0, 0, 0

            # print(R_x, R_y, R_z, zoom, shear)
            input = rotation3d_itk(input, R_x, R_y, R_z, zoom, shear)
            input = np.array(input)
            if target is not None:
                target = rotation3d_itk(target, R_x, R_y, R_z, zoom, shear)
                target = np.array(target)

                target[target >= 0.5] = 1.0
                target[target < 0.5] = 0.0
            scaling_ratio_x = zoom[0]
            scaling_ratio_y = zoom[1]
            scaling_ratio_z = zoom[2]

        else:
            scaling_ratio_x = 1.0
            scaling_ratio_y = 1.0
            scaling_ratio_z = 1.0

        if self.shift > 0:
            shift_x = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
            shift_y = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
            shift_z = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
        else:
            shift_x, shift_y, shift_z = 0, 0, 0

        crop_x = int(input.shape[0] / 2 - self.crop_size / 2 + shift_x)
        crop_y = int(input.shape[1] / 2 - self.crop_size / 2 + shift_y)
        crop_z = int(input.shape[2] / 2 - self.crop_size / 2 + shift_z)

        '''
                # visualization
        print('scaling ratio', scaling_ratio_xy, scaling_ratio_z)

        if not os.path.exists('/data2/botong/Github/save_img_dataset/'):
            os.makedirs('/data2/botong/Github/save_img_dataset/')
        for i in range(input.shape[2]):
            misc.imsave('/data2/botong/Github/save_img_dataset/input_%02d.png'%(i + 1), input[:,:,i])
            misc.imsave('/data2/botong/Github/save_img_dataset/target_%02d.png' % (i + 1), target[:, :, i])
        '''

        input_aug = input[crop_x: (crop_x + self.crop_size), crop_y: (crop_y + self.crop_size),
                    crop_z: (crop_z + self.crop_size)]
        if target is not None:
            target_aug = target[crop_x: (crop_x + self.crop_size), crop_y: (crop_y + self.crop_size),
                         crop_z: (crop_z + self.crop_size)]

        '''
                # visualization
        for i in range(input_aug.shape[2]):
            misc.imsave('/data2/botong/Github/save_img_dataset/input_aug_%02d.png'%(i + 1), input_aug[:,:,i])
            misc.imsave('/data2/botong/Github/save_img_dataset/target_aug_%02d.png' % (i + 1), target_aug[:, :, i])
        '''

        transpose_type = [(0, 1, 2), (1, 0, 2), (2, 0, 1)]

        if self.transpose:
            trans_idx = np.random.choice(3)
            if trans_idx != 0:
                input_aug = input_aug.transpose(transpose_type[trans_idx])
                if target is not None:
                    target_aug = target_aug.transpose(transpose_type[trans_idx])

        if self.flip:
            flip_x = np.random.choice(2) * 2 - 1
            flip_y = np.random.choice(2) * 2 - 1
            flip_z = np.random.choice(2) * 2 - 1
            input_aug = input_aug[::flip_x, ::flip_y, ::flip_z]
            if target is not None:
                target_aug = target_aug[::flip_x, ::flip_y, ::flip_z]

        if self.noise:
            input_aug = input_aug + np.random.normal(0, 0.1, (self.crop_size, self.crop_size, self.crop_size))

        if self.random_mask == 1:
            rand_x = np.random.choice(self.crop_size)
            rand_y = np.random.choice(self.crop_size)
            rand_z = np.random.choice(self.crop_size)

            input_aug[rand_x:min(rand_x + self.random_mask_size, self.crop_size - 1),
            rand_y:min(rand_y + self.random_mask_size, self.crop_size - 1),
            rand_z:min(rand_z + self.random_mask_size, self.crop_size - 1)] = input_aug.mean()

        elif self.random_mask == 2:
            rand_x = np.random.randint(self.crop_size)
            rand_y = np.random.randint(self.crop_size)
            rand_z = np.random.randint(self.crop_size)

            rand_x1 = np.clip(rand_x - self.random_mask_size / 2, 0, self.crop_size)
            rand_x2 = np.clip(rand_x + self.random_mask_size / 2, 0, self.crop_size)
            rand_y1 = np.clip(rand_y - self.random_mask_size / 2, 0, self.crop_size)
            rand_y2 = np.clip(rand_y + self.random_mask_size / 2, 0, self.crop_size)
            rand_z1 = np.clip(rand_z - self.random_mask_size / 2, 0, self.crop_size)
            rand_z2 = np.clip(rand_z + self.random_mask_size / 2, 0, self.crop_size)

            input_aug[rand_x1:rand_x2, rand_y1:rand_y2, rand_z1:rand_z2] = input_aug.mean()

        if target is not None:
            return input_aug, target_aug
        else:
            return input_aug

    def _get_random_params(self, offset_max, index=0):
        # np.random.seed(int(time.time() + 1e5 * index))

        rand_x = np.random.rand()
        rand_y = np.random.rand()
        rand_z = np.random.rand()
        rand_w = np.random.rand()
        rand_h = np.random.rand()
        rand_r = np.random.rand()
        rand_lr = np.random.rand()
        rand_td = np.random.rand()
        rand_r1 = np.random.rand()

        offset_x = int((rand_x * 2 - 1) * offset_max * 0.3)
        offset_y = int((rand_y * 2 - 1) * offset_max * 0.3)
        offset_z = int((rand_z * 2 - 1) * offset_max * 0.3)
        offset_w = int(((rand_w + 0.25) * 2 - 1) * offset_max * 2)
        offset_h = int(((rand_h + 0.25) * 2 - 1) * offset_max * 2)
        offset_d = int(((rand_h + 0.25) * 2 - 1) * offset_max * 2)
        # offset_w = int(((rand_w) * 2 - 1) * offset_max * 2)
        # offset_h = int(((rand_h) * 2 - 1) * offset_max * 2)
        rand_angle = int(rand_r * 360)

        return offset_x, offset_y, offset_z, offset_w, offset_h, offset_d, rand_angle, rand_lr, rand_td, rand_r1

    def __len__(self):
        return len(self.image_filenames)

class DatasetFromPerm_txt_boosting(data.Dataset):
    def __init__(self, pair_image_list, crop_size=64, load_size=96, crop_mode='center', flip=False, transpose=False,
                 scale=1 / 255.0, shift=2, aug=0, is_seg=0, is_class=1, channel_num=1, W=600, W2=40,
                 random_mask=False, random_mask_size=16, noise=0, concat=0, scaling=0, MS_list=None,
                 edge_target=0, offset_max=10, merge_label=0, attr_id=8, norm=0, pair_type=1,
                 rotation=1, opt=None, path = False):
        super(DatasetFromPerm_txt_boosting, self).__init__()

        with open(pair_image_list) as f:
            lines = f.readlines()

        min_num = 100
        for line in lines:
            if len(line.split(' ')) <= min_num:
                min_num = len(line.split(' '))

        self.dict = Big_dict
        self.norm = norm
        self.attr_id = attr_id
        self.rotation = rotation
        self.image_filenames = []
        for line in lines:
            self.image_filenames.append(line.split(' ')[0])

        self.label = []
        self.raw_pred = []
        self.pid_list = []
        self.merge = merge_label

        if min_num == 14 or min_num == 5 or min_num == 6 or min_num == 23 or min_num == 16:
            self.spacing_x = [float(line.split(' ')[3]) for line in lines]
            self.spacing_z = [float(line.split(' ')[4]) for line in lines]
            for line in lines:
                one_list = []
                self.raw_pred.append(float(line.split(' ')[-1].strip('\n')))
                if min_num > 7 and is_seg == 1:
                    self.pid_list.append(line.split(' ')[0].split('/')[7])
                if merge_label:
                    if is_class == 1:
                        one_list = [convert_label(float(line.split(' ')[5 + self.attr_id].strip('\n')), self.attr_id,
                                                  merge_label)]
                    elif is_class != 9 and is_class != 18:
                        if pair_type == 1:
                            pairs = [3, 4]
                        elif pair_type == 2:
                            pairs = [4, 5]
                        elif pair_type == 3:
                            pairs = [5, 8]
                        elif pair_type == 4:
                            pairs = [4, 6]
                        elif pair_type == 5:
                            pairs = [5, 6]
                        elif pair_type == 6:
                            pairs = [4, 5, 6]
                        elif pair_type == 7:
                            pairs = [6, 8]
                        elif pair_type == 10:
                            pairs = [0, 1]
                        elif pair_type == 11:
                            pairs = range(11)

                        for idx in pairs:
                            one_list.append(
                                convert_label(float(line.split(' ')[5 + idx].strip('\n')), idx, merge_label))
                    else:
                        for idx in range(is_class):
                            # print line.split(' ')[0]
                            # print is_class, len(line.split(' '))
                            one_list.append(
                                convert_label(float(line.split(' ')[5 + idx].strip('\n')), idx, merge_label))
                else:
                    if is_class == 1:
                        one_list = [int(round(float(line.split(' ')[5 + self.attr_id].strip('\n'))))]
                    else:
                        for idx in range(is_class):
                            # here all atribute adopt int
                            one_list.append(int(round(float(line.split(' ')[5 + idx].strip('\n')))))
                if is_class > 1:
                    self.label.append([one_list])
                elif is_class == 1:
                    self.label.append([one_list[-1]])  # ??
                else:
                    self.label = []
            self.seg_filenames = [line.strip().split(' ')[1] for line in lines]

            self.label = np.array(self.label)
            if np.min(self.label) == 1:
                self.label = self.label - 1
        elif min_num == 2:
            self.spacing_x = [0.6 for line in lines]
            self.spacing_z = [0.6 for line in lines]
            self.seg_filenames = [line.strip().split(' ')[1] for line in lines]

        else:
            raise ('sub_dir file is not valid')

        self.raw_pred = np.array(self.raw_pred)
        self.path = path
        self.W2 = W2
        self.crop_size = crop_size
        self.load_size = load_size
        self.crop_mode = crop_mode
        self.flip = flip
        self.transpose = transpose
        self.scale = scale
        self.shift = shift
        self.aug = aug
        self.is_seg = is_seg
        self.is_class = is_class
        self.channel_num = channel_num
        self.random_mask = random_mask
        self.random_mask_size = random_mask_size
        self.noise = noise
        self.concat = concat
        self.scaling = scaling
        self.weight = np.ones((len(self.image_filenames)))
        self.instance_loss = np.zeros((len(self.image_filenames)))
        self.MS_list = MS_list
        self.edge_target = edge_target
        self.offset_max = offset_max
        self.shear = 0

    def __getitem__(self, index):
        final_shape = (self.channel_num, self.crop_size, self.crop_size, self.crop_size)
        class_shape = (1, self.is_class)
        index_shape = (1, 1)
        table_shape = (1, 24)
        # print self.label[index]
        if self.aug == 1:
            input = self.scale * load_img_path(self.image_filenames[index]) #96 * 96 * 96

            position = self.image_filenames[index].find('mask') + 4
            P = self.image_filenames[index][:position] + '_PET' + self.image_filenames[index][position:] #pet path
            input_PET = self.scale * load_img_path(P)
            if self.norm:
                input = (input - 0.5) / 0.5
                input_PET = (input_PET - 0.5) / 0.5
            if np.where(input > 0)[0].shape[0] < 1 or np.where(input_PET > 0)[0].shape[0] < 1:
                print(self.image_filenames[index])
                print('bad input image with all blank')

            if self.is_seg == 1:
                # label = self.label[index]
                target = self.scale * load_img_path(self.seg_filenames[index])
                target[target >= 0.5] = 1
                target[target <= 0.5] = 0
                if np.where(target == 1)[0].shape[0] < 1:
                    print(self.seg_filenames[index])
                    print('bad target image with all blank')

                input, target = self._aug_data_seg(input, index, target)

                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:

                    if self.is_class:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(self.label[index]).reshape(class_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(index).reshape(index_shape).astype('int'))

            else:
                if self.concat == 1:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        print('bad target image with all blank')

                    input, target = self._aug_data_seg(input, index, target)
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:

                        print('bad target image with all blank')

                    nodule_size = compute_nodule_size(target)

                    input = self._aug_data_seg(input, index, nodule_size=None)
                    ######### For pet:::::::::::::


                    target = self.scale * load_img_path(P)
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print('bad target image with all blank')

                    nodule_size = compute_nodule_size(target)

                    input_PET = self._aug_data_seg(input_PET, index, nodule_size=None)
                    if self.merge == 8 or self.merge == 9:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('float32')), \
                               torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))
                    else:
                        if self.path:
                            return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                                np.array(self.label[index]).reshape(class_shape).astype('int')), torch.from_numpy(
                                np.array(index).reshape(index_shape).astype('int')), torch.from_numpy(
                                input_PET.astype('float32').reshape(final_shape)),[self.image_filenames[index]]

                        else:

                            return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('int')), torch.from_numpy(np.array(index).reshape(index_shape).astype('int')), torch.from_numpy(input_PET.astype('float32').reshape(final_shape))

        else:
            input = self.scale * load_img_path(self.image_filenames[index])
            position = self.image_filenames[index].find('mask') + 4
            P = self.image_filenames[index][:position] + '_PET' + self.image_filenames[index][position:]  # pet path
            input_PET = self.scale * load_img_path(P)
            if self.norm:
                input = (input - 0.5) / 0.5
                input_PET = (input_PET - 0.5) / 0.5
            if np.where(input > 0)[0].shape[0] < 1 or np.where(input_PET > 0)[0].shape[0] < 1:
                print(self.image_filenames[index])
                print('bad input image with all blank')

            if self.crop_mode == 'center':
                crop_x = int(input.shape[0] / 2 - self.crop_size / 2)
                crop_y = int(input.shape[0] / 2 - self.crop_size / 2)
                crop_z = int(input.shape[0] / 2 - self.crop_size / 2)

                input = input[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                        crop_x: (crop_x + self.crop_size)]

                crop_x = int(input_PET.shape[0] / 2 - self.crop_size / 2)
                crop_y = int(input_PET.shape[0] / 2 - self.crop_size / 2)
                crop_z = int(input_PET.shape[0] / 2 - self.crop_size / 2)

                input_PET = input_PET[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                        crop_x: (crop_x + self.crop_size)]

                if self.concat == 1:
                    target = self.scale * load_img_path(self.seg_filenames[index])
                    target[target >= 0.5] = 1
                    target[target <= 0.5] = 0
                    if np.where(target == 1)[0].shape[0] < 1:
                        print(self.seg_filenames[index])
                        raise ('bad target image with all blank')
                    target = target[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                             crop_x: (crop_x + self.crop_size)]



            if self.is_seg == 1:
                # 1. norm? or 1,0 ??  2.不用aug?
                target = self.scale * load_img_path(self.seg_filenames[index])
                target[target >= 0.5] = 1
                target[target <= 0.5] = 0
                if np.where(target == 1)[0].shape[0] < 1:
                    print(self.seg_filenames[index])
                    raise ('bad target image with all blank')

                if self.crop_mode == 'center':
                    target = target[crop_z: (crop_z + self.crop_size), crop_y: (crop_y + self.crop_size),
                             crop_x: (crop_x + self.crop_size)]

                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    if self.is_class:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(self.label[index]).reshape(class_shape).astype('int'))
                    else:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                            target.astype('float32').reshape(final_shape)), torch.from_numpy(
                            np.array(index).reshape(index_shape).astype('int'))

            else:
                if self.concat == 1:
                    use_input = np.concatenate([[input], [target]], axis=0)
                    return torch.from_numpy(use_input.astype('float32').reshape(final_shape)), torch.from_numpy(
                        np.array(self.label[index]).reshape(class_shape).astype('int'))
                else:
                    if self.merge == 8 or self.merge == 9:
                        return torch.from_numpy(input.astype('float32').reshape(final_shape)), \
                               torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('float32')), \
                               torch.from_numpy(np.array(index).reshape(index_shape).astype('int'))
                    else:
                        if self.path:
                            return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(
                                np.array(self.label[index]).reshape(class_shape).astype('int')), torch.from_numpy(
                                np.array(index).reshape(index_shape).astype('int')), torch.from_numpy(
                                input_PET.astype('float32').reshape(final_shape)), [self.image_filenames[index]]
                        else:
                            return torch.from_numpy(input.astype('float32').reshape(final_shape)), torch.from_numpy(np.array(self.label[index]).reshape(class_shape).astype('int')),torch.from_numpy(np.array(index).reshape(index_shape).astype('int')),torch.from_numpy(input_PET.astype('float32').reshape(final_shape))



    def _aug_data_seg(self, input, index, target=None, nodule_size=None):
        if self.offset_max:
            offset_x, offset_y, offset_z, offset_w, offset_h, offset_d, rand_angle, rand_lr, rand_td, rand_r1 = self._get_random_params(
                self.offset_max, index)

        ran_zoom = [self.crop_size, self.crop_size, self.crop_size]
        if nodule_size is not None:
            rand_w = np.random.rand()
            rand_h = np.random.rand()
            rand_d = np.random.rand()

            if max(nodule_size) <= 3:
                offset_w = rand_w * ran_zoom[0] / 2.0
                offset_h = rand_h * ran_zoom[1] / 2.0
                offset_d = rand_d * ran_zoom[2] / 2.0
            elif max(nodule_size) >= self.crop_size / 2:
                offset_w = - rand_w * ran_zoom[0] / 2.0
                offset_h = - rand_h * ran_zoom[1] / 2.0
                offset_d = - rand_d * ran_zoom[2] / 2.0

        ran_zoom[0] = ran_zoom[0] + offset_w
        ran_zoom[1] = ran_zoom[1] + offset_h
        ran_zoom[2] = ran_zoom[2] + offset_d
        zoom = [self.crop_size / float(ran_zoom[0]), self.crop_size / float(ran_zoom[1]),
                self.crop_size / float(ran_zoom[2])]
        # zoom is bigger means minimize the image

        if self.shear:
            hyx = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hzx = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hxy = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hzy = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hxz = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            hyz = (np.random.choice(2) * 2 - 1) * 0.2 * np.random.rand()
            shear = [hyx, hzx, hxy, hzy, hxz, hyz]
        else:
            shear = [0, 0, 0, 0, 0, 0]

        if self.scaling:
            # scaling_list = [2.0, 1.75, 1.5, 1.25, 1, 0.8]
            # scaling_ratio_xy = random.sample(scaling_list, 1)[0]
            # scaling_ratio_z = random.sample(scaling_list, 1)[0]

            '''
            # visualization
            if not os.path.exists('/data2/botong/Github/save_img_dataset/'):
                os.makedirs('/data2/botong/Github/save_img_dataset/')
            for i in range(input.shape[2]):
                misc.imsave('/data2/botong/Github/save_img_dataset/raw_input_%02d.png'%(i + 1), input[:,:,i])
                misc.imsave('/data2/botong/Github/save_img_dataset/raw_target_%02d.png' % (i + 1), target[:, :, i])
            '''

            # input = ndimage.zoom(input, (scaling_ratio_xy, scaling_ratio_xy, scaling_ratio_z), mode='wrap')
            # target = ndimage.zoom(target, (scaling_ratio_xy, scaling_ratio_xy, scaling_ratio_z), mode='wrap')
            if self.rotation == 1:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
            elif self.rotation == 2:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(15)
            elif self.rotation == 3:
                R_x = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_y = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
                R_z = (np.random.choice(2) * 2 - 1) * np.random.choice(360)
            else:
                R_x, R_y, R_z = 0, 0, 0

            # print(R_x, R_y, R_z, zoom, shear)
            input = rotation3d_itk(input, R_x, R_y, R_z, zoom, shear)
            input = np.array(input)
            if target is not None:
                target = rotation3d_itk(target, R_x, R_y, R_z, zoom, shear)
                target = np.array(target)

                target[target >= 0.5] = 1.0
                target[target < 0.5] = 0.0
            scaling_ratio_x = zoom[0]
            scaling_ratio_y = zoom[1]
            scaling_ratio_z = zoom[2]

        else:
            scaling_ratio_x = 1.0
            scaling_ratio_y = 1.0
            scaling_ratio_z = 1.0

        if self.shift > 0:
            shift_x = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
            shift_y = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
            shift_z = (np.random.choice(2) * 2 - 1) * np.random.choice(int(round(self.shift)))
        else:
            shift_x, shift_y, shift_z = 0, 0, 0

        crop_x = int(input.shape[0] / 2 - self.crop_size / 2 + shift_x)
        crop_y = int(input.shape[1] / 2 - self.crop_size / 2 + shift_y)
        crop_z = int(input.shape[2] / 2 - self.crop_size / 2 + shift_z)

        '''
                # visualization
        print('scaling ratio', scaling_ratio_xy, scaling_ratio_z)

        if not os.path.exists('/data2/botong/Github/save_img_dataset/'):
            os.makedirs('/data2/botong/Github/save_img_dataset/')
        for i in range(input.shape[2]):
            misc.imsave('/data2/botong/Github/save_img_dataset/input_%02d.png'%(i + 1), input[:,:,i])
            misc.imsave('/data2/botong/Github/save_img_dataset/target_%02d.png' % (i + 1), target[:, :, i])
        '''

        input_aug = input[crop_x: (crop_x + self.crop_size), crop_y: (crop_y + self.crop_size),
                    crop_z: (crop_z + self.crop_size)]
        if target is not None:
            target_aug = target[crop_x: (crop_x + self.crop_size), crop_y: (crop_y + self.crop_size),
                         crop_z: (crop_z + self.crop_size)]

        '''
                # visualization
        for i in range(input_aug.shape[2]):
            misc.imsave('/data2/botong/Github/save_img_dataset/input_aug_%02d.png'%(i + 1), input_aug[:,:,i])
            misc.imsave('/data2/botong/Github/save_img_dataset/target_aug_%02d.png' % (i + 1), target_aug[:, :, i])
        '''

        transpose_type = [(0, 1, 2), (1, 0, 2), (2, 0, 1)]

        if self.transpose:
            trans_idx = np.random.choice(3)
            if trans_idx != 0:
                input_aug = input_aug.transpose(transpose_type[trans_idx])
                if target is not None:
                    target_aug = target_aug.transpose(transpose_type[trans_idx])

        if self.flip:
            flip_x = np.random.choice(2) * 2 - 1
            flip_y = np.random.choice(2) * 2 - 1
            flip_z = np.random.choice(2) * 2 - 1
            input_aug = input_aug[::flip_x, ::flip_y, ::flip_z]
            if target is not None:
                target_aug = target_aug[::flip_x, ::flip_y, ::flip_z]

        if self.noise:
            input_aug = input_aug + np.random.normal(0, 0.1, (self.crop_size, self.crop_size, self.crop_size))

        if self.random_mask == 1:
            rand_x = np.random.choice(self.crop_size)
            rand_y = np.random.choice(self.crop_size)
            rand_z = np.random.choice(self.crop_size)

            input_aug[rand_x:min(rand_x + self.random_mask_size, self.crop_size - 1),
            rand_y:min(rand_y + self.random_mask_size, self.crop_size - 1),
            rand_z:min(rand_z + self.random_mask_size, self.crop_size - 1)] = input_aug.mean()

        elif self.random_mask == 2:
            rand_x = np.random.randint(self.crop_size)
            rand_y = np.random.randint(self.crop_size)
            rand_z = np.random.randint(self.crop_size)

            rand_x1 = np.clip(rand_x - self.random_mask_size / 2, 0, self.crop_size)
            rand_x2 = np.clip(rand_x + self.random_mask_size / 2, 0, self.crop_size)
            rand_y1 = np.clip(rand_y - self.random_mask_size / 2, 0, self.crop_size)
            rand_y2 = np.clip(rand_y + self.random_mask_size / 2, 0, self.crop_size)
            rand_z1 = np.clip(rand_z - self.random_mask_size / 2, 0, self.crop_size)
            rand_z2 = np.clip(rand_z + self.random_mask_size / 2, 0, self.crop_size)

            input_aug[rand_x1:rand_x2, rand_y1:rand_y2, rand_z1:rand_z2] = input_aug.mean()

        if target is not None:
            return input_aug, target_aug
        else:
            return input_aug

    def _get_random_params(self, offset_max, index=0):
        # np.random.seed(int(time.time() + 1e5 * index))

        rand_x = np.random.rand()
        rand_y = np.random.rand()
        rand_z = np.random.rand()
        rand_w = np.random.rand()
        rand_h = np.random.rand()
        rand_r = np.random.rand()
        rand_lr = np.random.rand()
        rand_td = np.random.rand()
        rand_r1 = np.random.rand()

        offset_x = int((rand_x * 2 - 1) * offset_max * 0.3)
        offset_y = int((rand_y * 2 - 1) * offset_max * 0.3)
        offset_z = int((rand_z * 2 - 1) * offset_max * 0.3)
        offset_w = int(((rand_w + 0.25) * 2 - 1) * offset_max * 2)
        offset_h = int(((rand_h + 0.25) * 2 - 1) * offset_max * 2)
        offset_d = int(((rand_h + 0.25) * 2 - 1) * offset_max * 2)
        # offset_w = int(((rand_w) * 2 - 1) * offset_max * 2)
        # offset_h = int(((rand_h) * 2 - 1) * offset_max * 2)
        rand_angle = int(rand_r * 360)

        return offset_x, offset_y, offset_z, offset_w, offset_h, offset_d, rand_angle, rand_lr, rand_td, rand_r1

    def __len__(self):
        return len(self.image_filenames)
import scipy.sparse as sp


def encode_onehot(labels):
    classes = set(labels)
    classes_dict = {c: np.identity(len(classes))[i, :] for i, c in
                    enumerate(classes)}
    labels_onehot = np.array(list(map(classes_dict.get, labels)),
                             dtype=np.int32)
    return labels_onehot


def normalize(mx):
    """Row-normalize sparse matrix"""
    rowsum = np.array(mx.sum(1))
    r_inv = np.power(rowsum, -1).flatten()
    r_inv[np.isinf(r_inv)] = 0.
    r_mat_inv = sp.diags(r_inv)
    mx = r_mat_inv.dot(mx)
    return mx


def accuracy(output, labels):
    preds = output.max(1)[1].type_as(labels)
    correct = preds.eq(labels).double()
    correct = correct.sum()
    return correct / len(labels)


def sparse_mx_to_torch_sparse_tensor(sparse_mx):
    """Convert a scipy sparse matrix to a torch sparse tensor."""
    sparse_mx = sparse_mx.tocoo().astype(np.float32)
    indices = torch.from_numpy(
        np.vstack((sparse_mx.row, sparse_mx.col)).astype(np.int64))
    values = torch.from_numpy(sparse_mx.data)
    shape = torch.Size(sparse_mx.shape)
    return torch.sparse.FloatTensor(indices, values, shape)


def rescale_L(L, lmax=2):
    """Rescale Laplacian eigenvalues to [-1,1]"""
    M, M = L.shape
    try:
        I = scipy.sparse.identity(M, format='csr', dtype=L.dtype)
    except:
        I = scipy.identity(M, format='csr', dtype=L.dtype)
    L = 2 * L / lmax

    L -= I
    return L


def lmax_L(L):
    """Compute largest Laplacian eigenvalue"""
    try:
        temp = scipy.sparse.linalg.eigsh(L, k=1, which='LM', return_eigenvectors=False)[0]
    except:
        temp = scipy.linalg.eigsh(L, k=1, which='LM', return_eigenvectors=False)[0]
    return temp


def load_adj_data(path="./data/", dataset="edges_2664", sym=1, norm=1, opt=None):
    """Load citation network dataset (cora only for now)"""
    # print('Loading {} dataset...'.format(dataset))

    # build graph
    if os.path.exists("{}{}.npy".format(path, dataset)):
        adj = np.load("{}{}.npy".format(path, dataset))
    elif dataset == 'edges_1111':
        adj = np.zeros((opt.class_num, opt.class_num))
    elif dataset == 'edges_9999':
        if 'GCN' in opt.model:
            adj = np.ones(opt.class_num) - np.eye(opt.class_num)
        else:
            adj = np.ones(opt.class_num)
    else:
        edges = np.genfromtxt("{}{}.txt".format(path, dataset),
                              dtype=np.int32)
        node_list = []
        for idx in range(edges.shape[0]):
            if edges[idx, 0] not in node_list:
                node_list.append(edges[idx, 0])
            if edges[idx, 1] not in node_list:
                node_list.append(edges[idx, 1])
        '''
        adj = sp.coo_matrix((np.ones(edges.shape[0]), (edges[:, 0], edges[:, 1])),
                            shape=(max(node_list)+1, max(node_list)+1),
                            dtype=np.float32)
    
        # build symmetric adjacency matrix
        adj = adj + adj.T.multiply(adj.T > adj) - adj.multiply(adj.T > adj)
        adj = normalize(adj + sp.eye(adj.shape[0]))
        adj = sparse_mx_to_torch_sparse_tensor(adj)
        '''
        adj = np.zeros((max(node_list) + 1, max(node_list) + 1))
        for idx in range(edges.shape[0]):
            adj[edges[idx, 0], edges[idx, 1]] = 1

    if 'GCN' in opt.model:
        # compute laplace matrix
        # print 'before: ', adj
        if sym == 1:
            adj = adj + adj.T * (adj.T > adj) - adj * (adj.T > adj)
        if norm == 1:
            adj = normalize(adj)
        # print 'normlized: ', adj
        adj = np.eye(adj.shape[0]) - adj
        # transform np.array to scipy.sparse
        # adj = scipy.sparse.csr_matrix(adj)
        lmax = lmax_L(adj)
        adj = rescale_L(adj, lmax)
    # print 'processed: ', adj
    else:
        if sym == 1:
            adj = adj + adj.T * (adj.T > adj) - adj * (adj.T > adj)

        if norm == 1:
            adj = normalize(adj + sp.eye(adj.shape[0]))
        else:
            adj = adj + sp.eye(adj.shape[0])

    # print(type(adj))
    if type(adj) == np.matrixlib.defmatrix.matrix:
        adj = torch.FloatTensor(adj)
    else:
        # print(adj)
        L = adj.tocoo()
        indices = np.column_stack((L.row, L.col)).T
        indices = indices.astype(np.int64)
        indices = torch.from_numpy(indices)
        indices = indices.type(torch.LongTensor)
        L_data = L.data.astype(np.float32)
        L_data = torch.from_numpy(L_data)
        L_data = L_data.type(torch.FloatTensor)
        adj = torch.sparse.FloatTensor(indices, L_data, torch.Size(L.shape))
    # adj = Variable(L, requires_grad=False)

    return adj
