# -*- coding: utf-8 -*-
from __future__ import print_function
import torch
import os
import numpy as np
import torch.nn as nn
import torch.optim as optim
from torch.autograd import Variable
import torch.nn.functional as F
from util.utils import *
from tqdm import tqdm
import time


def train_hgr(model, model_L, epoch, optimizer, optimizer_L,criterion, sample_train_loader, opt):
    print("HGR-----")
    batch_time = AverageMeter()
    losses = AverageMeter()
    segmentation = AverageMeter()
    losses_list = [AverageMeter() for i in range(opt.class_num)]
    classification = [AverageMeter() for i in range(opt.class_num)]

    all_pred = []
    all_pred_L = []
    all_labels = []m
    for idx in range(opt.class_num):
        all_pred.append(np.zeros((len(sample_train_loader) * 20, opt.class_config[idx])))
        all_pred_L.append(np.zeros((len(sample_train_loader) * 20, opt.class_config[idx])))
        all_labels.append(np.zeros(len(sample_train_loader) * 20))

    model.train()
    model_L.train()
    batch_start = 0
    start_time = time.time()
    loss= 0.0
    cnt = 0.0
    for iteration, batch in tqdm(enumerate(sample_train_loader, 1)):

        input, target, index, table = batch[0], batch[1], batch[2], batch[3]

        # print(torch.mean(input))

        if opt.class_num == 1 and len(np.where(target[:, :, 0] >= 0)[0]) == 0:
            continue
        if len(table) != 12:
            continue
        target_np = copy.deepcopy(target)
        input = input.cuda()
        target = target.cuda()
        table = table.cuda()

        batch_target = target
        optimizer.zero_grad()
        optimizer_L.zero_grad()

        _,batch_pred = model(x=input,feature=True)
        _,batch_pred_L = model_L(x=table,feature=True)
        loss_hgr =  opt.a * hgr_loss_function(batch_pred,batch_pred_L)
        loss_hgr.backward()
        loss += loss_hgr.item()
        cnt += 1
        optimizer.step()
        optimizer_L.step()

        batch_time.update(time.time() - start_time)
        start_time = time.time()
        losses.update(loss_hgr.item(), input.size(0))



        batch_start += input.size(0)

    #print("HGR average loss = ",loss/cnt)


# 训练过程
def train_mig(model, model_L, epoch, optimizer, optimizer_L,criterion, sample_train_loader, opt):
    print("MIG-----")
    batch_time = AverageMeter()
    losses = AverageMeter()
    segmentation = AverageMeter()
    losses_list = [AverageMeter() for i in range(opt.class_num)]
    classification = [AverageMeter() for i in range(opt.class_num)]

    all_pred = []
    all_pred_L = []
    all_labels = []
    for idx in range(opt.class_num):
        all_pred.append(np.zeros((len(sample_train_loader) * 20, opt.class_config[idx])))
        all_pred_L.append(np.zeros((len(sample_train_loader) * 20, opt.class_config[idx])))
        all_labels.append(np.zeros(len(sample_train_loader) * 20))

    model.train()
    model_L.train()
    batch_start = 0
    start_time = time.time()
    loss= 0.0
    cnt = 0.0
    for iteration, batch in tqdm(enumerate(sample_train_loader, 1)):

        input, target, index, table = batch[0], batch[1], batch[2], batch[3]

        # print(torch.mean(input))

        if opt.class_num == 1 and len(np.where(target[:, :, 0] >= 0)[0]) == 0:
            continue
        if len(table) != 12:
            continue
        target_np = copy.deepcopy(target)
        input = input.cuda()
        target = target.cuda()
        table = table.cuda()

        batch_target = target
        p = torch.FloatTensor([0.6,0.4]).cuda()
        optimizer.zero_grad()
        optimizer_L.zero_grad()

        batch_pred = model(x=input)
        batch_pred_L = model_L(x=table)
        loss_mig =  opt.a * mig_loss_function(F.softmax(batch_pred[0],1),F.softmax(batch_pred_L[0],1),p,batch_pred_L[0].size(0))
        loss_mig.backward()
        loss += loss_mig.item()
        cnt += 1
        optimizer.step()
        optimizer_L.step()

        batch_time.update(time.time() - start_time)
        start_time = time.time()
        losses.update(loss_mig.item(), input.size(0))

        for idx in range(opt.class_num):
            non_index = np.where(target_np[:, :, idx] >= 0)[0]
            if len(non_index) == 0:
                # print('all zero')
                continue
            #print(batch_pred[idx].size())
            #print("Input:", input.size(0))
            all_pred[idx][batch_start: batch_start + input.size(0), :] = batch_pred[idx].detach().cpu().numpy()
            all_pred_L[idx][batch_start: batch_start + input.size(0), :] = batch_pred_L[idx].detach().cpu().numpy()
            all_labels[idx][batch_start: batch_start + input.size(0)] = \
                batch_target[:, :, idx].detach().cpu().numpy().reshape((input.size(0),))

        batch_start += input.size(0)

    for idx in range(opt.class_num):
        all_pred[idx] = all_pred[idx][:batch_start]
        all_pred_L[idx] = all_pred_L[idx][:batch_start]
        all_labels[idx] = all_labels[idx][:batch_start]
    #print("MIG average loss = ",loss/cnt)
    return all_pred, all_labels, all_pred_L, losses


def train(model, model_L, epoch, optimizer, optimizer_L,criterion, sample_train_loader, opt):
    batch_time = AverageMeter()
    losses = AverageMeter()
    segmentation = AverageMeter()
    losses_list = [AverageMeter() for i in range(opt.class_num)]
    classification = [AverageMeter() for i in range(opt.class_num)]

    all_pred = []
    all_pred_L = []
    all_labels = []
    for idx in range(opt.class_num):
        all_pred.append(np.zeros((len(sample_train_loader) * opt.batchSize, opt.class_config[idx])))
        all_pred_L.append(np.zeros((len(sample_train_loader) * opt.batchSize, opt.class_config[idx])))
        all_labels.append(np.zeros(len(sample_train_loader) * opt.batchSize))

    model.train()
    model_L.train()
    batch_start = 0
    start_time = time.time()
    total_loss_mri = 0.0
    total_loss_pet = 0.0
    for iteration, batch in tqdm(enumerate(sample_train_loader, 1)):

        input, target, index, table = batch[0], batch[1], batch[2], batch[3]
        # print(torch.mean(input))

        if opt.class_num == 1 and len(np.where(target[:, :, 0] >= 0)[0]) == 0:
            continue

        target_np = copy.deepcopy(target)
        input = input.cuda()
        target = target.cuda()
        table = table.cuda()
        batch_target = target

        optimizer.zero_grad()
        optimizer_L.zero_grad()

        batch_pred = model(x=input)
        batch_pred_L = model_L(x=table)

        loss = criterion(batch_pred[0],batch_target[:, :, 0].contiguous().view(-1))

        loss.backward()
        optimizer.step()
        total_loss_mri += loss.data

        loss_L = criterion(batch_pred_L[0],batch_target[:, :, 0].contiguous().view(-1))
        loss_L.backward()
        optimizer_L.step()
        total_loss_pet += loss_L.data

        batch_time.update(time.time() - start_time)
        start_time = time.time()
        losses.update(loss.item(), input.size(0))

        for idx in range(opt.class_num):
            non_index = np.where(target_np[:, :, idx] >= 0)[0]
            if len(non_index) == 0:
                # print('all zero')
                continue

            all_pred[idx][batch_start: batch_start + input.size(0), :] = batch_pred[idx].detach().cpu().numpy()
            all_pred_L[idx][batch_start: batch_start + input.size(0), :] = batch_pred_L[idx].detach().cpu().numpy()
            all_labels[idx][batch_start: batch_start + input.size(0)] = \
                batch_target[:, :, idx].detach().cpu().numpy().reshape((input.size(0),))

        batch_start += input.size(0)

    for idx in range(opt.class_num):
        all_pred[idx] = all_pred[idx][:batch_start]
        all_pred_L[idx] = all_pred_L[idx][:batch_start]
        all_labels[idx] = all_labels[idx][:batch_start]
    #print("Epoch:{},Loss mri:{}, Loss pet:{}".format(epoch,total_loss_mri,total_loss_pet))
    return all_pred, all_labels, all_pred_L, losses

def train_hgr_combine(model, model_L, epoch, optimizer, optimizer_L,criterion, sample_train_loader, opt,model_logistic=None,optimizer_logistic=None):
    batch_time = AverageMeter()
    losses = AverageMeter()
    segmentation = AverageMeter()
    losses_list = [AverageMeter() for i in range(opt.class_num)]
    classification = [AverageMeter() for i in range(opt.class_num)]

    all_pred = []
    all_pred_L = []
    all_labels = []
    for idx in range(opt.class_num):
        all_pred.append(np.zeros((len(sample_train_loader) * opt.batchSize, opt.class_config[idx])))
        all_pred_L.append(np.zeros((len(sample_train_loader) * opt.batchSize, opt.class_config[idx])))
        all_labels.append(np.zeros(len(sample_train_loader) * opt.batchSize))

    model.train()
    model_L.train()
    batch_start = 0
    start_time = time.time()
    total_loss_mri = 0.0
    total_loss_pet = 0.0
    for iteration, batch in tqdm(enumerate(sample_train_loader, 1)):

        input, target, index, table = batch[0], batch[1], batch[2], batch[3]
        # print(torch.mean(input))

        if opt.class_num == 1 and len(np.where(target[:, :, 0] >= 0)[0]) == 0:
            continue

        target_np = copy.deepcopy(target)
        input = input.cuda()
        target = target.cuda()
        table = table.cuda()
        batch_target = target

        optimizer.zero_grad()
        optimizer_L.zero_grad()
        optimizer_logistic.zero_grad()
        if opt.hgr:
            _,batch_pred = model(x=input,feature=True)
            _,batch_pred_L = model_L(x=table,feature=True)
            f = torch.cat((batch_pred,batch_pred_L),1)
            batch_pred = model_logistic(f)
            loss = criterion(batch_pred[0], batch_target[:, :, 0].contiguous().view(-1))

            loss.backward()
            optimizer.step()
            optimizer_L.step()
            optimizer_logistic.step()
            batch_pred_L = batch_pred
            total_loss_mri += loss.data
        else:
            batch_pred = model(x=input)
            batch_pred_L = model_L(x=table)
            loss = criterion(batch_pred[0],batch_target[:, :, 0].contiguous().view(-1))

            loss.backward()
            optimizer.step()
            total_loss_mri += loss.data

            loss_L = criterion(batch_pred_L[0],batch_target[:, :, 0].contiguous().view(-1))
            loss_L.backward()
            optimizer_L.step()
            total_loss_pet += loss_L.data

        batch_time.update(time.time() - start_time)
        start_time = time.time()
        losses.update(loss.item(), input.size(0))

        for idx in range(opt.class_num):
            non_index = np.where(target_np[:, :, idx] >= 0)[0]
            if len(non_index) == 0:
                # print('all zero')
                continue

            all_pred[idx][batch_start: batch_start + input.size(0), :] = batch_pred[idx].detach().cpu().numpy()
            all_pred_L[idx][batch_start: batch_start + input.size(0), :] = batch_pred_L[idx].detach().cpu().numpy()
            all_labels[idx][batch_start: batch_start + input.size(0)] = \
                batch_target[:, :, idx].detach().cpu().numpy().reshape((input.size(0),))

        batch_start += input.size(0)

    for idx in range(opt.class_num):
        all_pred[idx] = all_pred[idx][:batch_start]
        all_pred_L[idx] = all_pred_L[idx][:batch_start]
        all_labels[idx] = all_labels[idx][:batch_start]
    print("Loss mri:{}, Loss pet:{}".format(total_loss_mri,total_loss_pet))
    return all_pred, all_labels, all_pred_L, losses
def train_30T(model, epoch, optimizer,criterion, sample_train_loader, opt):
    batch_time = AverageMeter()
    losses = AverageMeter()
    segmentation = AverageMeter()
    losses_list = [AverageMeter() for i in range(opt.class_num)]
    classification = [AverageMeter() for i in range(opt.class_num)]

    all_pred = []
    all_pred_L = []
    all_labels = []
    for idx in range(opt.class_num):
        all_pred.append(np.zeros((len(sample_train_loader) * opt.batchSize, opt.class_config[idx])))
        all_labels.append(np.zeros(len(sample_train_loader) * opt.batchSize))

    model.train()
    batch_start = 0
    start_time = time.time()
    for iteration, batch in tqdm(enumerate(sample_train_loader, 1)):

        input, target, index= batch[0], batch[1], batch[2]

        # print(torch.mean(input))

        if opt.class_num == 1 and len(np.where(target[:, :, 0] >= 0)[0]) == 0:
            continue
        target_np = copy.deepcopy(target)
        input = input.cuda()
        target = target.cuda()
        batch_target = target

        optimizer.zero_grad()


        batch_pred = model(x=input)

        loss = criterion(batch_pred[0],batch_target[:, :, 0].contiguous().view(-1))
        loss.backward()
        optimizer.step()

        batch_time.update(time.time() - start_time)
        start_time = time.time()
        losses.update(loss.item(), input.size(0))

        for idx in range(opt.class_num):
            non_index = np.where(target_np[:, :, idx] >= 0)[0]
            if len(non_index) == 0:
                # print('all zero')
                continue

            all_pred[idx][batch_start: batch_start + input.size(0), :] = batch_pred[idx].detach().cpu().numpy()
            all_labels[idx][batch_start: batch_start + input.size(0)] = \
                batch_target[:, :, idx].detach().cpu().numpy().reshape((input.size(0),))

        batch_start += input.size(0)

    for idx in range(opt.class_num):
        all_pred[idx] = all_pred[idx][:batch_start]
        all_labels[idx] = all_labels[idx][:batch_start]

    return all_pred, all_labels, losses

def test_model(model, model_L, epoch, criterion, testing_data_loader, opt, model_logistic=None,bio=False):
    model.eval()
    model_L.eval()
    batch_time = AverageMeter()
    losses = AverageMeter()
    segmentation = AverageMeter()
    losses_list = [AverageMeter() for i in range(opt.class_num)]
    classification = [AverageMeter() for i in range(opt.class_num)]

    all_pred = []
    all_pred_L = []
    all_labels = []
    for idx in range(opt.class_num):
        all_pred.append(np.zeros((len(testing_data_loader) * opt.batchSize, opt.class_config[idx])))
        all_pred_L.append(np.zeros((len(testing_data_loader) * opt.batchSize, opt.class_config[idx])))
        all_labels.append(np.zeros(len(testing_data_loader) * opt.testBatchSize))

    batch_start = 0
    start_time = time.time()

    # logger.info('model training: '+  str(model.training))
    path_collector = []
    with torch.no_grad():
        for iteration, batch in enumerate(testing_data_loader, 1):

            input, target, index, table, path = batch[0], batch[1], batch[2], batch[3], batch[4]
            #input, target, index, table = batch[0], batch[1], batch[2], batch[3]
            target_np = copy.deepcopy(target)
            input = input.cuda()
            target = target.cuda()
            table = table.cuda()
            for i in range(len(path)):
                path_collector += path[i]

            batch_target = target
            if opt.hgr:
                _,batch_pred = model(x=input,feature=True)
                _,batch_pred_L = model_L(x=table,feature=True)
                f = torch.cat((batch_pred,batch_pred_L),1)
                batch_pred = model_logistic(f)
                batch_pred_L = batch_pred
            else:
                batch_pred = model(x=input)
                batch_pred_L = model_L(x=table)


            loss = torch.FloatTensor([0.0]).cuda()

            if not bio:
                if opt.class_num:
                    if isinstance(batch_pred, list):
                        if len(batch_pred) == 1 or len(batch_pred) == opt.class_num:
                            non_index = np.where(target_np[:, :, idx] >= 0)[0]
                            if len(non_index) == 0:
                                # print('all zero')
                                continue
                            temp = criterion(batch_pred[idx][non_index],
                                             batch_target[non_index, :, idx].contiguous().view(-1))
                            loss = torch.add(loss, temp)

            batch_time.update(time.time() - start_time)
            start_time = time.time()
            losses.update(loss.item(), input.size(0))

            for idx in range(opt.class_num):
                non_index = np.where(target_np[:, :, idx] >= 0)[0]
                if len(non_index) == 0:
                    # print('all zero')
                    continue

                all_pred[idx][batch_start: batch_start + input.size(0), :] = batch_pred[idx].detach().cpu().numpy()
                all_pred_L[idx][batch_start: batch_start + input.size(0), :] = batch_pred_L[idx].detach().cpu().numpy()
                all_labels[idx][batch_start: batch_start + input.size(0)] = batch_target[:, :,
                                                                            idx].detach().cpu().numpy().reshape((input.size(0),))

            batch_start += input.size(0)

        for idx in range(opt.class_num):
            all_pred[idx] = all_pred[idx][:batch_start]
            all_pred_L[idx] = all_pred_L[idx][:batch_start]
            all_labels[idx] = all_labels[idx][:batch_start]

        return all_pred, all_labels, all_pred_L, losses, path_collector
